@@ -1,8 +1,7 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
 # portmdmg.tcl
-# $Id: portmdmg.tcl 101263 2013-01-07 13:54:16Z jmr@macports.org $
 #
-# Copyright (c) 2005, 2007-2011 The MacPorts Project
+# Copyright (c) 2005, 2007-2013 The MacPorts Project
 # Copyright (c) 2003-2004 Apple Inc.
 # All rights reserved.
 #
@@ -17,7 +16,7 @@
 # 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -36,7 +35,7 @@ package require portutil 1.0
 
 set org.macports.mdmg [target_new org.macports.mdmg portmdmg::mdmg_main]
 target_runtype ${org.macports.mdmg} always
-target_provides ${org.macports.mdmg} mdmg 
+target_provides ${org.macports.mdmg} mdmg
 target_requires ${org.macports.mdmg} mpkg
 
 namespace eval portmdmg {
@@ -45,38 +44,34 @@ namespace eval portmdmg {
 set_ui_prefix
 
 proc portmdmg::mdmg_main {args} {
-    global subport version revision package.destpath UI_PREFIX
+    global subport version revision UI_PREFIX
 
     ui_msg "$UI_PREFIX [format [msgcat::mc "Creating disk image for %s-%s"] ${subport} ${version}]"
 
     if {[getuid] == 0 && [geteuid] != 0} {
-		setegid 0; seteuid 0
+		seteuid 0; setegid 0
 	}
 
     return [package_mdmg $subport $version $revision]
 }
 
 proc portmdmg::package_mdmg {portname portversion portrevision} {
-    global UI_PREFIX package.destpath portpath
-    global os.platform os.arch os.version os.major
-    
-    if {[expr (${portrevision} > 0)]} {
-        set imagename "${portname}-${portversion}-${portrevision}"
-    } else {
-        set imagename "${portname}-${portversion}"
-    }
-    
+    global UI_PREFIX package.destpath portpath \
+           os.arch os.major
+
+    set imagename [portpkg::image_name ${portname} ${portversion} ${portrevision}]
+
     set tmp_image ${package.destpath}/${imagename}.tmp.dmg
     set final_image ${package.destpath}/${imagename}.dmg
-    set mpkgpath ${package.destpath}/${portname}-${portversion}.mpkg
-    
+    set mpkgpath [portmpkg::mpkg_path $portname $portversion $portrevision]
+
     if {[file readable $final_image] && ([file mtime ${final_image}] >= [file mtime ${portpath}/Portfile])} {
-        ui_msg "$UI_PREFIX [format [msgcat::mc "Disk Image for %s-%s is up-to-date"] ${portname} ${portversion}]"
+        ui_msg "$UI_PREFIX [format [msgcat::mc "Disk Image for %s version %s is up-to-date"] ${portname} ${portversion}]"
         return 0
     }
 
     # partition for .dmg
-    if {${os.major} >= 9 && ${os.arch} == "i386"} {
+    if {${os.major} >= 9 && ${os.arch} eq "i386"} {
         # GUID_partition_scheme
         set subdev 1
     } else {
@@ -91,16 +86,16 @@ proc portmdmg::package_mdmg {portname portversion portrevision} {
     }
 
     set hdiutil [findBinary hdiutil $portutil::autoconf::hdiutil_path]
-    if {[system "$hdiutil create -quiet -fs HFS+ -volname ${imagename} -srcfolder ${mpkgpath} ${tmp_image}"] != ""} {
+    if {[system "$hdiutil create -quiet -fs HFS+ -volname ${imagename} -srcfolder ${mpkgpath} ${tmp_image}"] ne ""} {
         return -code error [format [msgcat::mc "Failed to create temporary image: %s"] ${imagename}]
     }
-    if {[system "$hdiutil convert ${tmp_image} -format UDCO -o ${final_image} -quiet"] != ""} {
+    if {[system "$hdiutil convert ${tmp_image} -format UDCO -o ${final_image} -quiet"] ne ""} {
         return -code error [format [msgcat::mc "Failed to convert to final image: %s"] ${final_image}]
     }
-    if {[system "$hdiutil internet-enable -quiet -yes ${final_image}"] != ""} {
+    if {[system "$hdiutil internet-enable -quiet -yes ${final_image}"] ne ""} {
         return -code error [format [msgcat::mc "Failed to internet-enable: %s"] ${final_image}]
     }
     file delete -force "${tmp_image}"
-    
+
     return 0
 }
