@@ -78,7 +78,7 @@ proc portconfigure::should_add_stdlib {} {
 }
 proc portconfigure::should_add_cxx_abi {} {
     set is_oldos [expr {[option os.platform] eq "darwin" && [option os.major] < 10}]
-    set is_mp_gcc [string match *g++-mp-* [option configure.cxx]]
+    set is_mp_gcc [regexp {(^|/)g\+\+-mp-.*} [option configure.cxx]]
     return [expr {$is_oldos && $is_mp_gcc}]
 }
 proc portconfigure::construct_cxxflags {flags} {
@@ -100,7 +100,7 @@ proc portconfigure::stdlib_trace {opt action args} {
 }
 proc portconfigure::configure_get_cxx_stdlib {} {
     global cxx_stdlib configure.cxx
-    if {![string match *g++-mp-* ${configure.cxx}]} {
+    if {![regexp {(^|/)g\+\+-mp-.*} ${configure.cxx}]} {
         return $cxx_stdlib
     } else {
         return macports-libstdc++
@@ -119,11 +119,11 @@ default configure.cmd       ./configure
 default configure.nice      {${buildnicevalue}}
 default configure.dir       {${worksrcpath}}
 default autoreconf.dir      {${worksrcpath}}
-default autoreconf.args     {{--install --verbose}}
+default autoreconf.args     "--install --verbose"
 default autoconf.dir        {${worksrcpath}}
-default autoconf.args       {--verbose}
+default autoconf.args       --verbose
 default automake.dir        {${worksrcpath}}
-default automake.args       {--verbose}
+default automake.args       --verbose
 default xmkmf.cmd           xmkmf
 default xmkmf.dir           {${worksrcpath}}
 default use_configure       yes
@@ -206,11 +206,11 @@ options configure.optflags \
         configure.fflags configure.f90flags configure.fcflags \
         configure.classpath
 # compiler flags section
-default configure.optflags      {-Os}
+default configure.optflags      -Os
 default configure.cflags        {${configure.optflags}}
 default configure.objcflags     {${configure.optflags}}
 default configure.cppflags      {-I${prefix}/include}
-default configure.ldflags       {"-L${prefix}/lib -Wl,-headerpad_max_install_names"}
+default configure.ldflags       {-L${prefix}/lib -Wl,-headerpad_max_install_names}
 default configure.libs          {}
 default configure.fflags        {${configure.optflags}}
 default configure.f90flags      {${configure.optflags}}
@@ -246,7 +246,7 @@ options configure.universal_archs configure.universal_args \
         configure.universal_objcflags \
         configure.universal_cppflags configure.universal_ldflags
 default configure.universal_archs       {[portconfigure::choose_supported_archs ${universal_archs}]}
-default configure.universal_args        {--disable-dependency-tracking}
+default configure.universal_args        --disable-dependency-tracking
 default configure.universal_cflags      {[portconfigure::configure_get_universal_cflags]}
 default configure.universal_objcflags   {${configure.universal_cflags}}
 default configure.universal_cppflags    {}
@@ -334,8 +334,15 @@ proc portconfigure::configure_start {args} {
 # configure.universal_archs based on supported_archs and build_arch or
 # universal_archs
 proc portconfigure::choose_supported_archs {archs} {
-    global supported_archs
-    if {$supported_archs eq ""} {
+    global supported_archs configure.sdk_version
+    # macOS 10.14 SDK only supports one arch, x86_64
+    if {${configure.sdk_version} ne "" && [vercmp ${configure.sdk_version} 10.14] >= 0} {
+        if {"x86_64" in $archs && ($supported_archs eq "" || "x86_64" in $supported_archs)} {
+            return x86_64
+        } else {
+            return ""
+        }
+    } elseif {$supported_archs eq ""} {
         return $archs
     }
     set ret {}
