@@ -1,6 +1,5 @@
 # et:ts=4
 # portinstall.tcl
-# $Id$
 #
 # Copyright (c) 2002 - 2004 Apple Inc.
 # Copyright (c) 2004 Robert Shaw <rshaw@opendarwin.org>
@@ -35,6 +34,7 @@
 package provide portinstall 1.0
 package require portutil 1.0
 package require registry2 2.0
+package require machista 1.0
 
 set org.macports.install [target_new org.macports.install portinstall::install_main]
 target_provides ${org.macports.install} install
@@ -78,10 +78,11 @@ proc portinstall::install_start {args} {
 
 proc portinstall::create_archive {location archive.type} {
     global workpath destpath portpath subport version revision portvariants \
-           epoch os.platform PortInfo installPlist \
+           epoch configure.cxx_stdlib portinstall::actual_cxx_stdlib \
+           portinstall::file_is_binary portinstall::cxx_stdlib_overridden \
+           cxx_stdlib PortInfo installPlist \
            archive.env archive.cmd archive.pre_args archive.args \
-           archive.post_args archive.dir \
-           depends_fetch depends_extract depends_build depends_lib depends_run
+           archive.post_args archive.dir depends_lib depends_run
     set archive.env {}
     set archive.cmd {}
     set archive.pre_args {}
@@ -122,7 +123,13 @@ proc portinstall::create_archive {location archive.type} {
                 set archive.pre_args {-cvf}
                 if {[regexp {z2?$} ${archive.type}]} {
                     if {[regexp {bz2?$} ${archive.type}]} {
-                        set gzip "bzip2"
+                        if {![catch {binaryInPath lbzip2}]} {
+                            set gzip "lbzip2"
+                        } elseif {![catch {binaryInPath pbzip2}]} {
+                            set gzip "pbzip2"
+                        } else {
+                            set gzip "bzip2"
+                        }
                         set level 9
                     } elseif {[regexp {lz$} ${archive.type}]} {
                         set gzip "lzma"
@@ -260,6 +267,8 @@ proc portinstall::create_archive {location archive.type} {
          }
     }
 
+    set have_fileIsBinary [expr {[option os.platform] eq "darwin"}]
+    set binary_files {}
     # also save the contents for our own use later
     set installPlist {}
     set destpathLen [string length $destpath]
@@ -271,11 +280,21 @@ proc portinstall::create_archive {location archive.type} {
         set relpath [string range $fullpath $destpathLen+1 end]
         if {[string index $relpath 0] ne "+"} {
             puts $fd "$relpath"
-            lappend installPlist [file join [file separator] $relpath]
+            set abspath [file join [file separator] $relpath]
+            lappend installPlist $abspath
             if {[file isfile $fullpath]} {
                 ui_debug "checksum file: $fullpath"
                 set checksum [md5 file $fullpath]
                 puts $fd "@comment MD5:$checksum"
+                if {$have_fileIsBinary} {
+                    # test if (mach-o) binary
+                    set is_binary [fileIsBinary $fullpath]
+                    if {$is_binary} {
+                        lappend binary_files $fullpath
+                    }
+                    puts $fd "@comment binary:$is_binary"
+                    set portinstall::file_is_binary($abspath) $is_binary
+                }
             }
         } else {
             lappend control $relpath
@@ -285,6 +304,14 @@ proc portinstall::create_archive {location archive.type} {
         puts $fd "@ignore"
         puts $fd "$relpath"
     }
+    set portinstall::actual_cxx_stdlib [get_actual_cxx_stdlib $binary_files]
+    puts $fd "@cxx_stdlib ${portinstall::actual_cxx_stdlib}"
+    if {${portinstall::actual_cxx_stdlib} ne "none"} {
+        set portinstall::cxx_stdlib_overridden [expr {${configure.cxx_stdlib} ne $cxx_stdlib}]
+    } else {
+        set portinstall::cxx_stdlib_overridden 0
+    }
+    puts $fd "@cxx_stdlib_overridden ${portinstall::cxx_stdlib_overridden}"
     close $fd
 
     # Now create the archive
@@ -305,11 +332,10 @@ proc portinstall::extract_contents {location type} {
 }
 
 proc portinstall::install_main {args} {
-    global subport version portpath categories description long_description \
-    homepage depends_run package-install workdir workpath \
-    worksrcdir UI_PREFIX destroot revision maintainers user_options \
-    portvariants negated_variants targets depends_lib PortInfo epoch license \
-    os.platform os.major portarchivetype installPlist registry.path porturl
+    global subport version portpath depends_run revision user_options \
+    portvariants negated_variants depends_lib PortInfo epoch \
+    os.platform os.major portarchivetype installPlist registry.path porturl \
+    portinstall::file_is_binary portinstall::actual_cxx_stdlib portinstall::cxx_stdlib_overridden
 
     set oldpwd [pwd]
     if {$oldpwd eq ""} {
@@ -324,7 +350,12 @@ proc portinstall::install_main {args} {
         file rename -force $archive_path $install_dir
         set location [file join $install_dir [file tail $archive_path]]
         set current_archive_type [string range [file extension $location] 1 end]
-        set installPlist [extract_contents $location $current_archive_type]
+        set contents [extract_contents $location $current_archive_type]
+        set installPlist [lindex $contents 0]
+        array set portinstall::file_is_binary [lindex $contents 1]
+        set cxxinfo [extract_archive_metadata $location $current_archive_type cxx_info]
+        set portinstall::actual_cxx_stdlib [lindex $cxxinfo 0]
+        set portinstall::cxx_stdlib_overridden [lindex $cxxinfo 1]
     } else {
         # throws an error if an unsupported value has been configured
         archiveTypeIsSupported $portarchivetype
@@ -357,6 +388,14 @@ proc portinstall::install_main {args} {
         $regref os_platform ${os.platform}
         $regref os_major ${os.major}
         $regref archs [get_canonical_archs]
+        if {${portinstall::actual_cxx_stdlib} ne ""} {
+            $regref cxx_stdlib ${portinstall::actual_cxx_stdlib}
+            $regref cxx_stdlib_overridden ${portinstall::cxx_stdlib_overridden}
+        } else {
+            # no info in the archive
+            global configure.cxx_stdlib cxx_stdlib
+            $regref cxx_stdlib_overridden [expr {${configure.cxx_stdlib} ne $cxx_stdlib}]
+        }
         # Trick to have a portable GMT-POSIX epoch-based time.
         $regref date [expr {[clock scan now -gmt true] - [clock scan "1970-1-1 00:00:00" -gmt true]}]
         if {[info exists negated_variants]} {
@@ -374,6 +413,10 @@ proc portinstall::install_main {args} {
         if {[info exists installPlist]} {
             # register files
             $regref map $installPlist
+            foreach f [array names portinstall::file_is_binary] {
+                set fileref [registry::file open [$regref id] $f]
+                $fileref binary $portinstall::file_is_binary($f)
+            }
         }
 
         # store portfile
@@ -385,6 +428,7 @@ proc portinstall::install_main {args} {
         set portfile_reg_path ${portfile_reg_dir}/Portfile
         if {![file isfile $portfile_reg_path] || [file size $portfile_reg_path] != $portfile_size || [sha256 file $portfile_reg_path] ne $portfile_sha256} {
             file copy -force $portfile_path $portfile_reg_dir
+            file attributes $portfile_reg_path -permissions 0644
         }
         $regref portfile ${portfile_sha256}-${portfile_size}
 
@@ -393,7 +437,7 @@ proc portinstall::install_main {args} {
             foreach pg $PortInfo(portgroups) {
                 set pgname [lindex $pg 0]
                 set pgversion [lindex $pg 1]
-                set groupFile [getportresourcepath $porturl "port1.0/group/${pgname}-${pgversion}.tcl"]
+                set groupFile [lindex $pg 2]
                 if {[file isfile $groupFile]} {
                     set pgsha256 [sha256 file $groupFile]
                     set pgsize [file size $groupFile]
@@ -403,6 +447,7 @@ proc portinstall::install_main {args} {
                         file mkdir $pg_reg_dir
                         file copy -force $groupFile $pg_reg_dir
                     }
+                    file attributes $pg_reg_path -permissions 0644
                     $regref addgroup $pgname $pgversion $pgsha256 $pgsize
                 } else {
                     ui_debug "install_main: no portgroup ${pgname}-${pgversion}.tcl found"
